<?php
declare(strict_types=1);

namespace Inti\Media\Filesystem\Adapter;

use Inti\Image\Convert\ConvertInterface;
use Inti\Image\Optimize\OptimizeInterface;
use Inti\Image\Resize\ResizeInterface;
use League\Flysystem\Config;
use League\Flysystem\FileAttributes;
use League\Flysystem\FilesystemAdapter;
use League\Flysystem\FilesystemException;

/**
 * Class ImageUploadAdapter
 *
 * @package Inti\Media\Filesystem\Adapter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/warehouse
 */
final class ImageFilesystemAdapter implements FilesystemAdapter
{
    /** @var FilesystemAdapter */
    private $adapter;
    /** @var ConvertInterface */
    private $convert;
    /** @var OptimizeInterface */
    private $optimize;
    /** @var ResizeInterface */
    private $resize;

    /**
     * Konstruktor.
     *
     * @param FilesystemAdapter $adapter
     * @param ConvertInterface|null $convert
     * @param OptimizeInterface|null $optimize
     * @param ResizeInterface|null $resize
     */
    public function __construct(
        FilesystemAdapter $adapter,
        ConvertInterface $convert = null,
        OptimizeInterface $optimize = null,
        ResizeInterface $resize = null
    ) {
        $this->adapter = $adapter;
        $this->convert = $convert;
        $this->optimize = $optimize;
        $this->resize = $resize;
    }

    private function parsePath(string $path): array
    {
        preg_match('#^((thumb/)?(\d*)x(\d*)/)?(.+)$#', $path, $matches);

        return [
            'cropped' => $matches[2] ? true : false,
            'width' => $matches[3] ? intval($matches[3]) : null,
            'height' => $matches[4] ? intval($matches[4]) : null,
            'raw' => $matches[5],
        ];
    }

    private function generateResizedImage(string $path): void
    {
        if (!$this->adapter->fileExists($path)) {
            $parsed = $this->parsePath($path);

            if ($parsed['width'] || $parsed['height'] && $this->resize instanceof ResizeInterface) {
                $raw = $this->read($parsed['raw']);
                $resized = $this->resize->resize($raw, $parsed['width'], $parsed['height'], $parsed['cropped']);
                $this->write($path, $resized, new Config());
            }
        }
    }

    /**
     * @param string $path
     * @return bool
     * @throws FilesystemException
     */
    public function fileExists(string $path): bool
    {
        $parsed = $this->parsePath($path);
        return $this->adapter->fileExists($parsed['raw']);
    }

    /**
     * @param string $path
     * @param string $contents
     * @param Config $config
     * @throws FilesystemException
     */
    public function write(string $path, string $contents, Config $config): void
    {
        if ($this->convert instanceof ConvertInterface) {
            $contents = $this->convert->convert($contents, 'image/webp');
        }

        if ($this->optimize instanceof OptimizeInterface) {
            $contents = $this->optimize->optimize($contents);
        }

        $this->adapter->write($path, $contents, $config);
    }

    /**
     * @param string $path
     * @param resource $contents
     * @param Config $config
     * @throws FilesystemException
     */
    public function writeStream(string $path, $contents, Config $config): void
    {
        if ($this->convert instanceof ConvertInterface) {
            $contents = $this->convert->convertStream($contents, 'image/webp');
        }

        if ($this->optimize instanceof OptimizeInterface) {
            $contents = $this->optimize->optimizeStream($contents);
        }

        $this->adapter->writeStream($path, $contents, $config);
    }

    public function read(string $path): string
    {
        $this->generateResizedImage($path);
        return $this->adapter->read($path);
    }

    public function readStream(string $path)
    {
        $this->generateResizedImage($path);
        return $this->adapter->readStream($path);
    }

    public function delete(string $path): void
    {
        $this->adapter->delete($path);
    }

    public function deleteDirectory(string $path): void
    {
        $this->adapter->deleteDirectory($path);
    }

    public function createDirectory(string $path, Config $config): void
    {
        $this->adapter->createDirectory($path, $config);
    }

    public function setVisibility(string $path, string $visibility): void
    {
        $this->adapter->setVisibility($path, $visibility);
    }

    public function visibility(string $path): FileAttributes
    {
        return $this->adapter->visibility($path);
    }

    public function mimeType(string $path): FileAttributes
    {
        return $this->adapter->mimeType($path);
    }

    public function lastModified(string $path): FileAttributes
    {
        return $this->adapter->lastModified($path);
    }

    public function fileSize(string $path): FileAttributes
    {
        return $this->adapter->fileSize($path);
    }

    public function listContents(string $path, bool $deep): iterable
    {
        return $this->adapter->listContents($path, $deep);
    }

    public function move(string $source, string $destination, Config $config): void
    {
        $this->adapter->move($source, $destination, $config);
    }

    public function copy(string $source, string $destination, Config $config): void
    {
        $this->adapter->copy($source, $destination, $config);
    }
}
